"""Test the masking utility functions."""

import logging
from unittest.mock import patch

from mcp_atlassian.utils.logging import log_config_param, mask_sensitive


class TestMaskSensitive:
    """Test the _mask_sensitive function."""

    def test_none_value(self):
        """Test masking None value."""
        assert mask_sensitive(None) == "Not Provided"

    def test_short_value(self):
        """Test masking short value."""
        assert mask_sensitive("abc") == "***"
        assert mask_sensitive("abcdef") == "******"
        assert mask_sensitive("abcdefgh", keep_chars=4) == "********"

    def test_normal_value(self):
        """Test masking normal value."""
        assert mask_sensitive("abcdefghijkl", keep_chars=2) == "ab********kl"
        assert mask_sensitive("abcdefghijkl") == "abcd****ijkl"
        assert (
            mask_sensitive("abcdefghijklmnopqrstuvwxyz", keep_chars=5)
            == "abcde****************vwxyz"
        )


class TestLogConfigParam:
    """Test the _log_config_param function."""

    @patch("mcp_atlassian.utils.logging.logging.Logger")
    def test_normal_param(self, mock_logger):
        """Test logging normal parameter."""
        log_config_param(mock_logger, "Jira", "URL", "https://jira.example.com")
        mock_logger.info.assert_called_once_with("Jira URL: https://jira.example.com")

    @patch("mcp_atlassian.utils.logging.logging.Logger")
    def test_none_param(self, mock_logger):
        """Test logging None parameter."""
        log_config_param(mock_logger, "Jira", "Projects Filter", None)
        mock_logger.info.assert_called_once_with("Jira Projects Filter: Not Provided")

    @patch("mcp_atlassian.utils.logging.logging.Logger")
    def test_sensitive_param(self, mock_logger):
        """Test logging sensitive parameter."""
        log_config_param(
            mock_logger, "Jira", "API Token", "abcdefghijklmnop", sensitive=True
        )
        mock_logger.info.assert_called_once_with("Jira API Token: abcd********mnop")

    def test_log_config_param_masks_proxy_url(self, caplog):
        """Test that log_config_param masks credentials in proxy URLs when sensitive=True."""
        logger = logging.getLogger("test-proxy-logger")
        proxy_url = "socks5://user:pass@proxy.example.com:1080"
        with caplog.at_level(logging.INFO, logger="test-proxy-logger"):
            log_config_param(logger, "Jira", "SOCKS_PROXY", proxy_url, sensitive=True)
        # Should mask the middle part of the URL, not show user:pass
        assert any(
            rec.message.startswith("Jira SOCKS_PROXY: sock")
            and rec.message.endswith("1080")
            and "user:pass" not in rec.message
            for rec in caplog.records
        )



"""Tests for custom headers parsing functionality."""

from mcp_atlassian.utils.env import get_custom_headers


class TestParseCustomHeaders:
    """Test the parse_custom_headers function."""

    def test_empty_input(self, monkeypatch):
        """Test parse_custom_headers with empty/None inputs."""
        # Test unset environment variable
        monkeypatch.delenv("TEST_HEADERS", raising=False)
        assert get_custom_headers("TEST_HEADERS") == {}

        # Test empty string
        monkeypatch.setenv("TEST_HEADERS", "")
        assert get_custom_headers("TEST_HEADERS") == {}

        # Test whitespace only
        monkeypatch.setenv("TEST_HEADERS", "   ")
        assert get_custom_headers("TEST_HEADERS") == {}

        monkeypatch.setenv("TEST_HEADERS", "\t\n")
        assert get_custom_headers("TEST_HEADERS") == {}

    def test_single_header(self, monkeypatch):
        """Test parsing a single header."""
        monkeypatch.setenv("TEST_HEADERS", "X-Custom=value123")
        result = get_custom_headers("TEST_HEADERS")
        assert result == {"X-Custom": "value123"}

        # Test with spaces around key and value
        monkeypatch.setenv("TEST_HEADERS", " X-Spaced = value with spaces ")
        result = get_custom_headers("TEST_HEADERS")
        assert result == {"X-Spaced": "value with spaces"}

    def test_multiple_headers(self, monkeypatch):
        """Test parsing multiple comma-separated headers."""
        monkeypatch.setenv("TEST_HEADERS", "X-Corp-Auth=token123,X-Dept=engineering")
        result = get_custom_headers("TEST_HEADERS")
        expected = {"X-Corp-Auth": "token123", "X-Dept": "engineering"}
        assert result == expected

    def test_headers_with_spaces(self, monkeypatch):
        """Test parsing headers with various spacing."""
        monkeypatch.setenv("TEST_HEADERS", " X-Key = value , X-Another = value2 ")
        result = get_custom_headers("TEST_HEADERS")
        expected = {"X-Key": "value", "X-Another": "value2"}
        assert result == expected

    def test_value_with_equals_signs(self, monkeypatch):
        """Test parsing headers where values contain equals signs."""
        monkeypatch.setenv("TEST_HEADERS", "X-Token=abc=def=123")
        result = get_custom_headers("TEST_HEADERS")
        assert result == {"X-Token": "abc=def=123"}

        # Multiple headers with equals in values
        monkeypatch.setenv(
            "TEST_HEADERS", "X-Token=abc=def,X-URL=https://api.example.com/v1?key=value"
        )
        result = get_custom_headers("TEST_HEADERS")
        expected = {
            "X-Token": "abc=def",
            "X-URL": "https://api.example.com/v1?key=value",
        }
        assert result == expected

    def test_malformed_headers(self, monkeypatch):
        """Test handling of malformed header strings."""
        # Header without equals sign - should be skipped
        monkeypatch.setenv("TEST_HEADERS", "invalid-header-format")
        result = get_custom_headers("TEST_HEADERS")
        assert result == {}

        # Mix of valid and invalid headers
        monkeypatch.setenv(
            "TEST_HEADERS", "X-Valid=value,invalid-header,X-Another=value2"
        )
        result = get_custom_headers("TEST_HEADERS")
        expected = {"X-Valid": "value", "X-Another": "value2"}
        assert result == expected

    def test_empty_key_or_value(self, monkeypatch):
        """Test handling of empty keys or values."""
        # Empty key - should be skipped
        monkeypatch.setenv("TEST_HEADERS", "=value")
        result = get_custom_headers("TEST_HEADERS")
        assert result == {}

        # Empty value - should be included
        monkeypatch.setenv("TEST_HEADERS", "X-Empty=")
        result = get_custom_headers("TEST_HEADERS")
        assert result == {"X-Empty": ""}

        # Whitespace-only key - should be skipped
        monkeypatch.setenv("TEST_HEADERS", "  =value")
        result = get_custom_headers("TEST_HEADERS")
        assert result == {}

        # Mix of empty and valid
        monkeypatch.setenv("TEST_HEADERS", "=empty_key,X-Valid=value, =another_empty")
        result = get_custom_headers("TEST_HEADERS")
        assert result == {"X-Valid": "value"}

    def test_special_characters_in_values(self, monkeypatch):
        """Test headers with special characters in values."""
        monkeypatch.setenv("TEST_HEADERS", "X-Special=!@#$%^&*()_+-[]{}|;':\"/<>?")
        result = get_custom_headers("TEST_HEADERS")
        assert result == {"X-Special": "!@#$%^&*()_+-[]{}|;':\"/<>?"}

    def test_unicode_characters(self, monkeypatch):
        """Test headers with unicode characters."""
        monkeypatch.setenv("TEST_HEADERS", "X-Unicode=café,X-Emoji=🚀")
        result = get_custom_headers("TEST_HEADERS")
        expected = {"X-Unicode": "café", "X-Emoji": "🚀"}
        assert result == expected

    def test_empty_pairs_in_list(self, monkeypatch):
        """Test handling of empty pairs in comma-separated list."""
        # Empty pairs should be skipped
        monkeypatch.setenv("TEST_HEADERS", "X-First=value1,,X-Second=value2,")
        result = get_custom_headers("TEST_HEADERS")
        expected = {"X-First": "value1", "X-Second": "value2"}
        assert result == expected

        # Only commas
        monkeypatch.setenv("TEST_HEADERS", ",,,")
        result = get_custom_headers("TEST_HEADERS")
        assert result == {}

    def test_complex_real_world_example(self, monkeypatch):
        """Test a complex real-world example."""
        headers_string = (
            "Authorization=Bearer eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9,"
            "X-API-Key=sk-1234567890abcdef,"
            "X-Request-ID=req_123456789,"
            "X-Custom-Header=value with spaces and = signs,"
            "User-Agent=MyApp/1.0 (Custom Agent)"
        )

        monkeypatch.setenv("TEST_HEADERS", headers_string)
        result = get_custom_headers("TEST_HEADERS")
        expected = {
            "Authorization": "Bearer eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9",
            "X-API-Key": "sk-1234567890abcdef",
            "X-Request-ID": "req_123456789",
            "X-Custom-Header": "value with spaces and = signs",
            "User-Agent": "MyApp/1.0 (Custom Agent)",
        }
        assert result == expected

    def test_case_sensitive_keys(self, monkeypatch):
        """Test that header keys are case-sensitive."""
        monkeypatch.setenv(
            "TEST_HEADERS", "x-lower=value1,X-UPPER=value2,X-Mixed=value3"
        )
        result = get_custom_headers("TEST_HEADERS")
        expected = {"x-lower": "value1", "X-UPPER": "value2", "X-Mixed": "value3"}
        assert result == expected

    def test_duplicate_keys(self, monkeypatch):
        """Test behavior with duplicate keys - later values should override."""
        monkeypatch.setenv("TEST_HEADERS", "X-Duplicate=first,X-Duplicate=second")
        result = get_custom_headers("TEST_HEADERS")
        assert result == {"X-Duplicate": "second"}

    def test_newlines_and_tabs_in_input(self, monkeypatch):
        """Test handling of newlines and tabs in input."""
        # These should be treated as part of values, not separators
        monkeypatch.setenv(
            "TEST_HEADERS", "X-Multi=line1\nline2,X-Tab=value\twith\ttabs"
        )
        result = get_custom_headers("TEST_HEADERS")
        expected = {"X-Multi": "line1\nline2", "X-Tab": "value\twith\ttabs"}
        assert result == expected



"""Lifecycle management utilities for graceful shutdown and signal handling."""

import logging
import signal
import sys
import threading
from typing import Any

logger = logging.getLogger("mcp-atlassian.utils.lifecycle")

# Global shutdown event for signal-safe handling
_shutdown_event = threading.Event()


def setup_signal_handlers() -> None:
    """Set up signal handlers for graceful shutdown.

    Registers handlers for SIGTERM, SIGINT, and SIGPIPE (if available) to ensure
    the application shuts down cleanly when receiving termination signals.

    This is particularly important for Docker containers running with the -i flag,
    which need to properly handle shutdown signals from parent processes.
    """

    def signal_handler(signum: int, frame: Any) -> None:
        """Handle shutdown signals gracefully.

        Uses event-based shutdown to avoid signal safety issues.
        Signal handlers should be minimal and avoid complex operations.
        """
        # Only safe operations in signal handlers - set the shutdown event
        _shutdown_event.set()

    # Register signal handlers
    signal.signal(signal.SIGTERM, signal_handler)
    signal.signal(signal.SIGINT, signal_handler)

    # Handle SIGPIPE which occurs when parent process closes the pipe
    try:
        signal.signal(signal.SIGPIPE, signal_handler)
        logger.debug("SIGPIPE handler registered")
    except AttributeError:
        # SIGPIPE may not be available on all platforms (e.g., Windows)
        logger.debug("SIGPIPE not available on this platform")


def ensure_clean_exit() -> None:
    """Ensure all output streams are flushed before exit.

    This is important for containerized environments where output might be
    buffered and could be lost if not properly flushed before exit.

    Handles cases where streams may already be closed by the parent process,
    particularly on Windows or when run as a child process.
    """
    logger.info("Server stopped, flushing output streams...")

    # Safely flush stdout
    try:
        if hasattr(sys.stdout, "closed") and not sys.stdout.closed:
            sys.stdout.flush()
    except (ValueError, OSError, AttributeError) as e:
        # Stream might be closed or redirected
        logger.debug(f"Could not flush stdout: {e}")

    # Safely flush stderr
    try:
        if hasattr(sys.stderr, "closed") and not sys.stderr.closed:
            sys.stderr.flush()
    except (ValueError, OSError, AttributeError) as e:
        # Stream might be closed or redirected
        logger.debug(f"Could not flush stderr: {e}")

    logger.debug("Output streams flushed, exiting gracefully")



"""Confluence API integration module.

This module provides access to Confluence content through the Model Context Protocol.
"""

from .client import ConfluenceClient
from .comments import CommentsMixin
from .config import ConfluenceConfig
from .labels import LabelsMixin
from .pages import PagesMixin
from .search import SearchMixin
from .spaces import SpacesMixin
from .users import UsersMixin


class ConfluenceFetcher(
    SearchMixin, SpacesMixin, PagesMixin, CommentsMixin, LabelsMixin, UsersMixin
):
    """Main entry point for Confluence operations, providing backward compatibility.

    This class combines functionality from various mixins to maintain the same
    API as the original ConfluenceFetcher class.
    """

    pass


__all__ = ["ConfluenceFetcher", "ConfluenceConfig", "ConfluenceClient"]



"""Base test classes and utilities for Better Confluence MCP tests."""

from unittest.mock import AsyncMock, MagicMock

import pytest


class BaseMixinTest:
    """Base class for mixin tests with common setup patterns."""

    @pytest.fixture
    def mock_config(self):
        """Mock configuration for testing."""
        return MagicMock()

    @pytest.fixture
    def mock_client(self):
        """Mock client with common methods."""
        client = MagicMock()
        # Add common client methods
        client.get = AsyncMock()
        client.post = AsyncMock()
        client.put = AsyncMock()
        client.delete = AsyncMock()
        return client


class BaseAuthTest:
    """Base class for authentication-related tests."""

    @pytest.fixture
    def basic_auth_env_vars(self):
        """Standard basic auth environment variables."""
        return {
            "CONFLUENCE_URL": "https://test.atlassian.net/wiki",
            "CONFLUENCE_USERNAME": "test@example.com",
            "CONFLUENCE_API_TOKEN": "test-token",
        }


class BaseServerTest:
    """Base class for server-related tests."""

    @pytest.fixture
    def mock_request(self):
        """Mock FastMCP request object."""
        request = MagicMock()
        request.state = MagicMock()
        return request

    @pytest.fixture
    def mock_context(self):
        """Mock FastMCP context object."""
        return MagicMock()



"""Unit tests for the SearchMixin class."""

from unittest.mock import MagicMock, patch

import pytest
import requests
from requests import HTTPError

from mcp_atlassian.confluence.search import SearchMixin
from mcp_atlassian.confluence.utils import quote_cql_identifier_if_needed
from mcp_atlassian.exceptions import MCPAtlassianAuthenticationError


class TestSearchMixin:
    """Tests for the SearchMixin class."""

    @pytest.fixture
    def search_mixin(self, confluence_client):
        """Create a SearchMixin instance for testing."""
        # SearchMixin inherits from ConfluenceClient, so we need to create it properly
        with patch(
            "mcp_atlassian.confluence.search.ConfluenceClient.__init__"
        ) as mock_init:
            mock_init.return_value = None
            mixin = SearchMixin()
            # Copy the necessary attributes from our mocked client
            mixin.confluence = confluence_client.confluence
            mixin.config = confluence_client.config
            mixin.preprocessor = confluence_client.preprocessor
            return mixin

    def test_search_success(self, search_mixin):
        """Test search with successful results."""
        # Prepare the mock
        search_mixin.confluence.cql.return_value = {
            "results": [
                {
                    "content": {
                        "id": "123456789",
                        "title": "Test Page",
                        "type": "page",
                        "space": {"key": "SPACE", "name": "Test Space"},
                        "version": {"number": 1},
                    },
                    "excerpt": "Test content excerpt",
                    "url": "https://confluence.example.com/pages/123456789",
                }
            ]
        }

        # Mock the preprocessor to return processed content
        search_mixin.preprocessor.process_html_content.return_value = (
            "<p>Processed HTML</p>",
            "Processed content",
        )

        # Call the method
        result = search_mixin.search("test query")

        # Verify API call
        search_mixin.confluence.cql.assert_called_once_with(cql="test query", limit=10)

        # Verify result
        assert len(result) == 1
        assert result[0].id == "123456789"
        assert result[0].title == "Test Page"
        assert result[0].content == "Processed content"

    def test_search_with_empty_results(self, search_mixin):
        """Test handling of empty search results."""
        # Mock an empty result set
        search_mixin.confluence.cql.return_value = {"results": []}

        # Act
        results = search_mixin.search("empty query")

        # Assert
        assert isinstance(results, list)
        assert len(results) == 0

    def test_search_with_non_page_content(self, search_mixin):
        """Test handling of non-page content in search results."""
        # Mock search results with non-page content
        search_mixin.confluence.cql.return_value = {
            "results": [
                {
                    "content": {"type": "blogpost", "id": "12345"},
                    "title": "Blog Post",
                    "excerpt": "This is a blog post",
                    "url": "/pages/12345",
                    "resultGlobalContainer": {"title": "TEST"},
                }
            ]
        }

        # Act
        results = search_mixin.search("blogpost query")

        # Assert
        assert isinstance(results, list)
        # The method should still handle them as pages since we're using models
        assert len(results) > 0

    def test_search_key_error(self, search_mixin):
        """Test handling of KeyError in search results."""
        # Mock a response missing required keys
        search_mixin.confluence.cql.return_value = {"incomplete": "data"}

        # Act
        results = search_mixin.search("invalid query")

        # Assert
        assert isinstance(results, list)
        assert len(results) == 0

    def test_search_request_exception(self, search_mixin):
        """Test handling of RequestException during search."""
        # Mock a network error
        search_mixin.confluence.cql.side_effect = requests.RequestException("API error")

        # Act
        results = search_mixin.search("error query")

        # Assert
        assert isinstance(results, list)
        assert len(results) == 0

    def test_search_value_error(self, search_mixin):
        """Test handling of ValueError during search."""
        # Mock a value error
        search_mixin.confluence.cql.side_effect = ValueError("Value error")

        # Act
        results = search_mixin.search("error query")

        # Assert
        assert isinstance(results, list)
        assert len(results) == 0

    def test_search_type_error(self, search_mixin):
        """Test handling of TypeError during search."""
        # Mock a type error
        search_mixin.confluence.cql.side_effect = TypeError("Type error")

        # Act
        results = search_mixin.search("error query")

        # Assert
        assert isinstance(results, list)
        assert len(results) == 0

    def test_search_with_spaces_filter(self, search_mixin):
        """Test searching with spaces filter from parameter."""
        # Prepare the mock
        search_mixin.confluence.cql.return_value = {
            "results": [
                {
                    "content": {
                        "id": "123456789",
                        "title": "Test Page",
                        "type": "page",
                        "space": {"key": "SPACE", "name": "Test Space"},
                        "version": {"number": 1},
                    },
                    "excerpt": "Test content excerpt",
                    "url": "https://confluence.example.com/pages/123456789",
                }
            ]
        }

        # Mock the preprocessor
        search_mixin.preprocessor.process_html_content.return_value = (
            "<p>Processed HTML</p>",
            "Processed content",
        )

        # Test with single space filter
        result = search_mixin.search("test query", spaces_filter="DEV")

        # Verify space was properly quoted in the CQL query
        quoted_dev = quote_cql_identifier_if_needed("DEV")
        search_mixin.confluence.cql.assert_called_with(
            cql=f"(test query) AND (space = {quoted_dev})",
            limit=10,
        )
        assert len(result) == 1

        # Test with multiple spaces filter
        result = search_mixin.search("test query", spaces_filter="DEV,TEAM")

        # Verify spaces were properly quoted in the CQL query
        quoted_dev = quote_cql_identifier_if_needed("DEV")
        quoted_team = quote_cql_identifier_if_needed("TEAM")
        search_mixin.confluence.cql.assert_called_with(
            cql=f"(test query) AND (space = {quoted_dev} OR space = {quoted_team})",
            limit=10,
        )
        assert len(result) == 1

        # Test with filter when query already has space
        result = search_mixin.search('space = "EXISTING"', spaces_filter="DEV")
        search_mixin.confluence.cql.assert_called_with(
            cql='space = "EXISTING"',  # Should not add filter when space already exists
            limit=10,
        )
        assert len(result) == 1

    def test_search_with_config_spaces_filter(self, search_mixin):
        """Test search using spaces filter from config."""
        # Prepare the mock
        search_mixin.confluence.cql.return_value = {
            "results": [
                {
                    "content": {
                        "id": "123456789",
                        "title": "Test Page",
                        "type": "page",
                        "space": {"key": "SPACE", "name": "Test Space"},
                        "version": {"number": 1},
                    },
                    "excerpt": "Test content excerpt",
                    "url": "https://confluence.example.com/pages/123456789",
                }
            ]
        }

        # Mock the preprocessor
        search_mixin.preprocessor.process_html_content.return_value = (
            "<p>Processed HTML</p>",
            "Processed content",
        )

        # Set config filter
        search_mixin.config.spaces_filter = "DEV,TEAM"

        # Test with config filter
        result = search_mixin.search("test query")

        # Verify spaces were properly quoted in the CQL query
        quoted_dev = quote_cql_identifier_if_needed("DEV")
        quoted_team = quote_cql_identifier_if_needed("TEAM")
        search_mixin.confluence.cql.assert_called_with(
            cql=f"(test query) AND (space = {quoted_dev} OR space = {quoted_team})",
            limit=10,
        )
        assert len(result) == 1

        # Test that explicit filter overrides config filter
        result = search_mixin.search("test query", spaces_filter="OVERRIDE")

        # Verify space was properly quoted in the CQL query
        quoted_override = quote_cql_identifier_if_needed("OVERRIDE")
        search_mixin.confluence.cql.assert_called_with(
            cql=f"(test query) AND (space = {quoted_override})",
            limit=10,
        )
        assert len(result) == 1

    def test_search_general_exception(self, search_mixin):
        """Test handling of general exceptions during search."""
        # Mock a general exception
        search_mixin.confluence.cql.side_effect = Exception("General error")

        # Act
        results = search_mixin.search("error query")

        # Assert
        assert isinstance(results, list)
        assert len(results) == 0

    def test_search_user_success(self, search_mixin):
        """Test search_user with successful results."""
        # Prepare the mock response
        search_mixin.confluence.get.return_value = {
            "results": [
                {
                    "user": {
                        "type": "known",
                        "accountId": "1234asdf",
                        "accountType": "atlassian",
                        "email": "first.last@example.com",
                        "publicName": "First Last",
                        "displayName": "First Last",
                        "isExternalCollaborator": False,
                        "profilePicture": {
                            "path": "/wiki/aa-avatar/1234asdf",
                            "width": 48,
                            "height": 48,
                            "isDefault": False,
                        },
                    },
                    "title": "First Last",
                    "excerpt": "",
                    "url": "/people/1234asdf",
                    "entityType": "user",
                    "lastModified": "2025-06-02T13:35:59.680Z",
                    "score": 0.0,
                }
            ],
            "start": 0,
            "limit": 25,
            "size": 1,
            "totalSize": 1,
            "cqlQuery": "( user.fullname ~ 'First Last' )",
            "searchDuration": 115,
        }

        # Call the method
        result = search_mixin.search_user('user.fullname ~ "First Last"')

        # Verify API call
        search_mixin.confluence.get.assert_called_once_with(
            "rest/api/search/user",
            params={"cql": 'user.fullname ~ "First Last"', "limit": 10},
        )

        # Verify result
        assert len(result) == 1
        assert result[0].user.account_id == "1234asdf"
        assert result[0].user.display_name == "First Last"
        assert result[0].user.email == "first.last@example.com"
        assert result[0].title == "First Last"
        assert result[0].entity_type == "user"

    def test_search_user_with_empty_results(self, search_mixin):
        """Test search_user with empty results."""
        # Mock an empty result set
        search_mixin.confluence.get.return_value = {
            "results": [],
            "start": 0,
            "limit": 25,
            "size": 0,
            "totalSize": 0,
            "cqlQuery": 'user.fullname ~ "Nonexistent"',
            "searchDuration": 50,
        }

        # Act
        results = search_mixin.search_user('user.fullname ~ "Nonexistent"')

        # Assert
        assert isinstance(results, list)
        assert len(results) == 0

    def test_search_user_with_custom_limit(self, search_mixin):
        """Test search_user with custom limit."""
        # Prepare the mock response
        search_mixin.confluence.get.return_value = {
            "results": [],
            "start": 0,
            "limit": 5,
            "size": 0,
            "totalSize": 0,
            "cqlQuery": 'user.fullname ~ "Test"',
            "searchDuration": 30,
        }

        # Call with custom limit
        search_mixin.search_user('user.fullname ~ "Test"', limit=5)

        # Verify API call with correct limit
        search_mixin.confluence.get.assert_called_once_with(
            "rest/api/search/user", params={"cql": 'user.fullname ~ "Test"', "limit": 5}
        )

    @pytest.mark.parametrize(
        "exception_type,exception_args,expected_result",
        [
            (requests.RequestException, ("Network error",), []),
            (ValueError, ("Value error",), []),
            (TypeError, ("Type error",), []),
            (Exception, ("General error",), []),
            (KeyError, ("Missing key",), []),
        ],
    )
    def test_search_user_exception_handling(
        self, search_mixin, exception_type, exception_args, expected_result
    ):
        """Test search_user handling of various exceptions that return empty list."""
        # Mock the exception
        search_mixin.confluence.get.side_effect = exception_type(*exception_args)

        # Act
        results = search_mixin.search_user('user.fullname ~ "Test"')

        # Assert
        assert isinstance(results, list)
        assert results == expected_result

    @pytest.mark.parametrize(
        "status_code,exception_type",
        [
            (401, MCPAtlassianAuthenticationError),
            (403, MCPAtlassianAuthenticationError),
        ],
    )
    def test_search_user_http_auth_errors(
        self, search_mixin, status_code, exception_type
    ):
        """Test search_user handling of HTTP authentication errors."""
        # Mock HTTP error
        mock_response = MagicMock()
        mock_response.status_code = status_code
        http_error = HTTPError(f"HTTP {status_code}")
        http_error.response = mock_response
        search_mixin.confluence.get.side_effect = http_error

        # Act and assert
        with pytest.raises(exception_type):
            search_mixin.search_user('user.fullname ~ "Test"')

    def test_search_user_http_other_error(self, search_mixin):
        """Test search_user handling of other HTTP errors."""
        # Mock HTTP 500 error
        mock_response = MagicMock()
        mock_response.status_code = 500
        http_error = HTTPError("Internal Server Error")
        http_error.response = mock_response
        search_mixin.confluence.get.side_effect = http_error

        # Act and assert - should re-raise the HTTPError
        with pytest.raises(HTTPError):
            search_mixin.search_user('user.fullname ~ "Test"')

    @pytest.mark.parametrize(
        "mock_response,expected_length",
        [
            ({"incomplete": "data"}, 0),  # KeyError case
            (None, 0),  # None response case
            ({"results": []}, 0),  # Empty results case
        ],
    )
    def test_search_user_edge_cases(self, search_mixin, mock_response, expected_length):
        """Test search_user handling of edge cases in API responses."""
        search_mixin.confluence.get.return_value = mock_response

        # Act
        results = search_mixin.search_user('user.fullname ~ "Test"')

        # Assert
        assert isinstance(results, list)
        assert len(results) == expected_length

    # You can also parametrize the regular search method exception tests:
    @pytest.mark.parametrize(
        "exception_type,exception_args,expected_result",
        [
            (requests.RequestException, ("API error",), []),
            (ValueError, ("Value error",), []),
            (TypeError, ("Type error",), []),
            (Exception, ("General error",), []),
            (KeyError, ("Missing key",), []),
        ],
    )
    def test_search_exception_handling(
        self, search_mixin, exception_type, exception_args, expected_result
    ):
        """Test search handling of various exceptions that return empty list."""
        # Mock the exception
        search_mixin.confluence.cql.side_effect = exception_type(*exception_args)

        # Act
        results = search_mixin.search("error query")

        # Assert
        assert isinstance(results, list)
        assert results == expected_result

    # Parametrize CQL query tests:
    @pytest.mark.parametrize(
        "query,limit,expected_params",
        [
            (
                'user.fullname ~ "Test"',
                10,
                {"cql": 'user.fullname ~ "Test"', "limit": 10},
            ),
            (
                'user.email ~ "test@example.com"',
                5,
                {"cql": 'user.email ~ "test@example.com"', "limit": 5},
            ),
            (
                'user.fullname ~ "John" AND user.email ~ "@company.com"',
                15,
                {
                    "cql": 'user.fullname ~ "John" AND user.email ~ "@company.com"',
                    "limit": 15,
                },
            ),
        ],
    )
    def test_search_user_api_parameters(
        self, search_mixin, query, limit, expected_params
    ):
        """Test that search_user calls the API with correct parameters."""
        # Mock successful response
        search_mixin.confluence.get.return_value = {
            "results": [],
            "start": 0,
            "limit": limit,
            "totalSize": 0,
        }

        # Act
        search_mixin.search_user(query, limit=limit)

        # Assert API was called with correct parameters
        search_mixin.confluence.get.assert_called_once_with(
            "rest/api/search/user", params=expected_params
        )

    def test_search_user_with_complex_cql_query(self, search_mixin):
        """Test search_user with complex CQL query containing operators."""
        # Mock successful response
        search_mixin.confluence.get.return_value = {
            "results": [],
            "start": 0,
            "limit": 10,
            "totalSize": 0,
        }

        complex_query = 'user.fullname ~ "John" AND user.email ~ "@company.com" OR user.displayName ~ "JD"'

        # Act
        search_mixin.search_user(complex_query)

        # Assert API was called with the exact query
        search_mixin.confluence.get.assert_called_once_with(
            "rest/api/search/user", params={"cql": complex_query, "limit": 10}
        )

    def test_search_user_result_processing(self, search_mixin):
        """Test that search_user properly processes and returns user search result objects."""
        # Mock response with user data
        search_mixin.confluence.get.return_value = {
            "results": [
                {
                    "user": {
                        "accountId": "test-account-id",
                        "displayName": "Test User",
                        "email": "test@example.com",
                        "isExternalCollaborator": False,
                    },
                    "title": "Test User",
                    "entityType": "user",
                    "score": 1.5,
                }
            ],
            "start": 0,
            "limit": 10,
            "totalSize": 1,
        }

        # Act
        results = search_mixin.search_user('user.fullname ~ "Test User"')

        # Assert result structure
        assert len(results) == 1
        assert hasattr(results[0], "user")
        assert hasattr(results[0], "title")
        assert hasattr(results[0], "entity_type")
        assert results[0].user.account_id == "test-account-id"
        assert results[0].user.display_name == "Test User"
        assert results[0].title == "Test User"
        assert results[0].entity_type == "user"



"""Unit tests for the Confluence users module."""

from unittest.mock import MagicMock, patch

import pytest
from requests.exceptions import HTTPError

from mcp_atlassian.confluence.users import UsersMixin
from mcp_atlassian.exceptions import MCPAtlassianAuthenticationError


class TestUsersMixin:
    """Tests for the UsersMixin class."""

    @pytest.fixture
    def users_mixin(self, confluence_client):
        """Create a UsersMixin instance for testing."""
        # UsersMixin inherits from ConfluenceClient, so we need to create it properly
        with patch(
            "mcp_atlassian.confluence.users.ConfluenceClient.__init__"
        ) as mock_init:
            mock_init.return_value = None
            mixin = UsersMixin()
            # Copy the necessary attributes from our mocked client
            mixin.confluence = confluence_client.confluence
            mixin.config = confluence_client.config
            return mixin

    # Mock user data for different scenarios
    @pytest.fixture
    def mock_user_data_cloud(self):
        """Mock user data for Confluence Cloud."""
        return {
            "accountId": "5b10ac8d82e05b22cc7d4ef5",
            "accountType": "atlassian",
            "email": "user@example.com",
            "publicName": "Test User",
            "displayName": "Test User",
            "profilePicture": {
                "path": "/wiki/aa-avatar/5b10ac8d82e05b22cc7d4ef5",
                "width": 48,
                "height": 48,
                "isDefault": False,
            },
            "isExternalCollaborator": False,
            "accountStatus": "active",
        }

    @pytest.fixture
    def mock_user_data_server(self):
        """Mock user data for Confluence Server/DC."""
        return {
            "username": "testuser",
            "userKey": "testuser-key-12345",
            "displayName": "Test User",
            "fullName": "Test User Full Name",
            "email": "testuser@example.com",
            "status": "active",
        }

    @pytest.fixture
    def mock_user_data_with_status(self):
        """Mock user data with status expansion."""
        return {
            "accountId": "5b10ac8d82e05b22cc7d4ef5",
            "accountType": "atlassian",
            "email": "user@example.com",
            "publicName": "Test User",
            "displayName": "Test User",
            "accountStatus": "active",
            "status": "Active",  # Expanded status field
        }

    @pytest.fixture
    def mock_current_user_data(self):
        """Mock current user data for get_current_user_info."""
        return {
            "accountId": "5b10ac8d82e05b22cc7d4ef5",
            "type": "known",
            "accountType": "atlassian",
            "email": "current@example.com",
            "publicName": "Current User",
            "displayName": "Current User",
            "profilePicture": {
                "path": "/wiki/aa-avatar/5b10ac8d82e05b22cc7d4ef5",
                "width": 48,
                "height": 48,
                "isDefault": False,
            },
            "isExternalCollaborator": False,
            "isGuest": False,
            "locale": "en_US",
            "accountStatus": "active",
        }

    def test_get_user_details_by_accountid_success(
        self, users_mixin, mock_user_data_cloud
    ):
        """Test successfully getting user details by account ID."""
        # Arrange
        account_id = "5b10ac8d82e05b22cc7d4ef5"
        users_mixin.confluence.get_user_details_by_accountid.return_value = (
            mock_user_data_cloud
        )

        # Act
        result = users_mixin.get_user_details_by_accountid(account_id)

        # Assert
        users_mixin.confluence.get_user_details_by_accountid.assert_called_once_with(
            account_id, None
        )
        assert result == mock_user_data_cloud
        assert result["accountId"] == account_id
        assert result["displayName"] == "Test User"

    def test_get_user_details_by_accountid_with_expand(
        self, users_mixin, mock_user_data_with_status
    ):
        """Test getting user details by account ID with status expansion."""
        # Arrange
        account_id = "5b10ac8d82e05b22cc7d4ef5"
        expand = "status"
        users_mixin.confluence.get_user_details_by_accountid.return_value = (
            mock_user_data_with_status
        )

        # Act
        result = users_mixin.get_user_details_by_accountid(account_id, expand=expand)

        # Assert
        users_mixin.confluence.get_user_details_by_accountid.assert_called_once_with(
            account_id, expand
        )
        assert result == mock_user_data_with_status
        assert result["status"] == "Active"
        assert result["accountStatus"] == "active"

    def test_get_user_details_by_accountid_invalid_account_id(self, users_mixin):
        """Test getting user details with invalid account ID."""
        # Arrange
        invalid_account_id = "invalid-account-id"
        users_mixin.confluence.get_user_details_by_accountid.side_effect = Exception(
            "User not found"
        )

        # Act/Assert
        with pytest.raises(Exception, match="User not found"):
            users_mixin.get_user_details_by_accountid(invalid_account_id)

    def test_get_user_details_by_username_success(
        self, users_mixin, mock_user_data_server
    ):
        """Test successfully getting user details by username."""
        # Arrange
        username = "testuser"
        users_mixin.confluence.get_user_details_by_username.return_value = (
            mock_user_data_server
        )

        # Act
        result = users_mixin.get_user_details_by_username(username)

        # Assert
        users_mixin.confluence.get_user_details_by_username.assert_called_once_with(
            username, None
        )
        assert result == mock_user_data_server
        assert result["username"] == username
        assert result["displayName"] == "Test User"

    def test_get_user_details_by_username_with_expand(
        self, users_mixin, mock_user_data_server
    ):
        """Test getting user details by username with status expansion."""
        # Arrange
        username = "testuser"
        expand = "status"
        mock_data_with_status = mock_user_data_server.copy()
        mock_data_with_status["status"] = "Active"
        users_mixin.confluence.get_user_details_by_username.return_value = (
            mock_data_with_status
        )

        # Act
        result = users_mixin.get_user_details_by_username(username, expand=expand)

        # Assert
        users_mixin.confluence.get_user_details_by_username.assert_called_once_with(
            username, expand
        )
        assert result == mock_data_with_status
        assert result["status"] == "Active"

    def test_get_user_details_by_username_invalid_username(self, users_mixin):
        """Test getting user details with invalid username."""
        # Arrange
        invalid_username = "nonexistent-user"
        users_mixin.confluence.get_user_details_by_username.side_effect = Exception(
            "User not found"
        )

        # Act/Assert
        with pytest.raises(Exception, match="User not found"):
            users_mixin.get_user_details_by_username(invalid_username)

    def test_get_user_details_by_username_server_dc_pattern(
        self, users_mixin, mock_user_data_server
    ):
        """Test that username lookup follows Server/DC patterns."""
        # Arrange
        username = "dc.user@example.com"  # Email-like username common in DC
        users_mixin.confluence.get_user_details_by_username.return_value = (
            mock_user_data_server
        )

        # Act
        result = users_mixin.get_user_details_by_username(username)

        # Assert
        users_mixin.confluence.get_user_details_by_username.assert_called_once_with(
            username, None
        )
        assert result == mock_user_data_server

    def test_get_current_user_info_success(self, users_mixin, mock_current_user_data):
        """Test successfully getting current user info."""
        # Arrange
        users_mixin.confluence.get.return_value = mock_current_user_data

        # Act
        result = users_mixin.get_current_user_info()

        # Assert
        users_mixin.confluence.get.assert_called_once_with("rest/api/user/current")
        assert result == mock_current_user_data
        assert result["accountId"] == "5b10ac8d82e05b22cc7d4ef5"
        assert result["displayName"] == "Current User"

    def test_get_current_user_info_returns_non_dict(self, users_mixin):
        """Test get_current_user_info when API returns non-dict data."""
        # Arrange
        users_mixin.confluence.get.return_value = "Invalid response"

        # Act/Assert
        with pytest.raises(
            MCPAtlassianAuthenticationError,
            match="Confluence token validation failed: Did not receive valid JSON user data",
        ):
            users_mixin.get_current_user_info()

        users_mixin.confluence.get.assert_called_once_with("rest/api/user/current")

    def test_get_current_user_info_returns_none(self, users_mixin):
        """Test get_current_user_info when API returns None."""
        # Arrange
        users_mixin.confluence.get.return_value = None

        # Act/Assert
        with pytest.raises(
            MCPAtlassianAuthenticationError,
            match="Confluence token validation failed: Did not receive valid JSON user data",
        ):
            users_mixin.get_current_user_info()

    def test_get_current_user_info_http_error_401(self, users_mixin):
        """Test get_current_user_info with 401 authentication error."""
        # Arrange
        mock_response = MagicMock()
        mock_response.status_code = 401
        http_error = HTTPError(response=mock_response)
        users_mixin.confluence.get.side_effect = http_error

        # Act/Assert
        with pytest.raises(
            MCPAtlassianAuthenticationError,
            match="Confluence token validation failed: 401 from /rest/api/user/current",
        ):
            users_mixin.get_current_user_info()

    def test_get_current_user_info_http_error_403(self, users_mixin):
        """Test get_current_user_info with 403 forbidden error."""
        # Arrange
        mock_response = MagicMock()
        mock_response.status_code = 403
        http_error = HTTPError(response=mock_response)
        users_mixin.confluence.get.side_effect = http_error

        # Act/Assert
        with pytest.raises(
            MCPAtlassianAuthenticationError,
            match="Confluence token validation failed: 403 from /rest/api/user/current",
        ):
            users_mixin.get_current_user_info()

    def test_get_current_user_info_http_error_other(self, users_mixin):
        """Test get_current_user_info with other HTTP error codes."""
        # Arrange
        mock_response = MagicMock()
        mock_response.status_code = 500
        http_error = HTTPError(response=mock_response)
        users_mixin.confluence.get.side_effect = http_error

        # Act/Assert
        with pytest.raises(
            MCPAtlassianAuthenticationError,
            match="Confluence token validation failed with HTTPError",
        ):
            users_mixin.get_current_user_info()

    def test_get_current_user_info_http_error_no_response(self, users_mixin):
        """Test get_current_user_info with HTTPError but no response object."""
        # Arrange
        http_error = HTTPError()
        http_error.response = None
        users_mixin.confluence.get.side_effect = http_error

        # Act/Assert
        with pytest.raises(
            MCPAtlassianAuthenticationError,
            match="Confluence token validation failed with HTTPError",
        ):
            users_mixin.get_current_user_info()

    def test_get_current_user_info_generic_exception(self, users_mixin):
        """Test get_current_user_info with generic exception."""
        # Arrange
        users_mixin.confluence.get.side_effect = ConnectionError("Network error")

        # Act/Assert
        with pytest.raises(
            MCPAtlassianAuthenticationError,
            match="Confluence token validation failed: Network error",
        ):
            users_mixin.get_current_user_info()

    @pytest.mark.parametrize(
        "expand_param",
        [
            None,
            "status",
            "",  # Empty string
        ],
    )
    def test_get_user_details_by_accountid_expand_parameter_handling(
        self, users_mixin, mock_user_data_cloud, expand_param
    ):
        """Test that expand parameter is properly handled for account ID lookup."""
        # Arrange
        account_id = "5b10ac8d82e05b22cc7d4ef5"
        expected_data = mock_user_data_cloud.copy()
        if expand_param == "status":
            expected_data["status"] = "Active"

        users_mixin.confluence.get_user_details_by_accountid.return_value = (
            expected_data
        )

        # Act
        result = users_mixin.get_user_details_by_accountid(account_id, expand_param)

        # Assert
        users_mixin.confluence.get_user_details_by_accountid.assert_called_once_with(
            account_id, expand_param
        )
        assert result == expected_data

    @pytest.mark.parametrize(
        "expand_param",
        [
            None,
            "status",
            "",  # Empty string
        ],
    )
    def test_get_user_details_by_username_expand_parameter_handling(
        self, users_mixin, mock_user_data_server, expand_param
    ):
        """Test that expand parameter is properly handled for username lookup."""
        # Arrange
        username = "testuser"
        expected_data = mock_user_data_server.copy()
        if expand_param == "status":
            expected_data["status"] = "Active"

        users_mixin.confluence.get_user_details_by_username.return_value = expected_data

        # Act
        result = users_mixin.get_user_details_by_username(username, expand_param)

        # Assert
        users_mixin.confluence.get_user_details_by_username.assert_called_once_with(
            username, expand_param
        )
        assert result == expected_data

    def test_users_mixin_inheritance(self, users_mixin):
        """Test that UsersMixin properly inherits from ConfluenceClient."""
        # Verify that UsersMixin is indeed a ConfluenceClient
        from mcp_atlassian.confluence.client import ConfluenceClient

        assert isinstance(users_mixin, ConfluenceClient)

        # Verify it has the expected attributes from ConfluenceClient
        assert hasattr(users_mixin, "confluence")
        assert hasattr(users_mixin, "config")

    def test_users_mixin_has_required_methods(self):
        """Test that UsersMixin has all required methods."""
        # Verify the mixin has the expected methods
        assert hasattr(UsersMixin, "get_user_details_by_accountid")
        assert hasattr(UsersMixin, "get_user_details_by_username")
        assert hasattr(UsersMixin, "get_current_user_info")

        # Verify method signatures
        import inspect

        # Check get_user_details_by_accountid signature
        sig = inspect.signature(UsersMixin.get_user_details_by_accountid)
        params = list(sig.parameters.keys())
        assert "self" in params
        assert "account_id" in params
        assert "expand" in params
        assert sig.parameters["expand"].default is None

        # Check get_user_details_by_username signature
        sig = inspect.signature(UsersMixin.get_user_details_by_username)
        params = list(sig.parameters.keys())
        assert "self" in params
        assert "username" in params
        assert "expand" in params
        assert sig.parameters["expand"].default is None

        # Check get_current_user_info signature
        sig = inspect.signature(UsersMixin.get_current_user_info)
        params = list(sig.parameters.keys())
        assert "self" in params
        assert len(params) == 1  # Only self parameter

    def test_user_permission_scenarios(self, users_mixin):
        """Test various permission error scenarios."""
        # Test 401 Unauthorized
        mock_response_401 = MagicMock()
        mock_response_401.status_code = 401
        http_error_401 = HTTPError(response=mock_response_401)
        users_mixin.confluence.get_user_details_by_accountid.side_effect = (
            http_error_401
        )

        with pytest.raises(Exception):  # Should propagate the original exception
            users_mixin.get_user_details_by_accountid("test-account-id")

        # Test 403 Forbidden
        mock_response_403 = MagicMock()
        mock_response_403.status_code = 403
        http_error_403 = HTTPError(response=mock_response_403)
        users_mixin.confluence.get_user_details_by_username.side_effect = http_error_403

        with pytest.raises(Exception):  # Should propagate the original exception
            users_mixin.get_user_details_by_username("testuser")

    def test_cloud_vs_server_authentication_patterns(self, users_mixin):
        """Test that different authentication patterns work for Cloud vs Server/DC."""
        # Mock Cloud response (account ID based)
        cloud_user_data = {
            "accountId": "5b10ac8d82e05b22cc7d4ef5",
            "accountType": "atlassian",
            "displayName": "Cloud User",
            "accountStatus": "active",
        }

        # Mock Server/DC response (username based)
        server_user_data = {
            "username": "serveruser",
            "userKey": "serveruser-key-12345",
            "displayName": "Server User",
            "status": "active",
        }

        # Test Cloud pattern
        users_mixin.confluence.get_user_details_by_accountid.return_value = (
            cloud_user_data
        )
        cloud_result = users_mixin.get_user_details_by_accountid(
            "5b10ac8d82e05b22cc7d4ef5"
        )
        assert cloud_result["accountId"] == "5b10ac8d82e05b22cc7d4ef5"
        assert "accountType" in cloud_result

        # Test Server/DC pattern
        users_mixin.confluence.get_user_details_by_username.return_value = (
            server_user_data
        )
        server_result = users_mixin.get_user_details_by_username("serveruser")
        assert server_result["username"] == "serveruser"
        assert "userKey" in server_result

    def test_response_data_validation_and_transformation(
        self, users_mixin, mock_user_data_cloud
    ):
        """Test that response data is properly validated and returned as-is."""
        # Arrange
        account_id = "5b10ac8d82e05b22cc7d4ef5"
        users_mixin.confluence.get_user_details_by_accountid.return_value = (
            mock_user_data_cloud
        )

        # Act
        result = users_mixin.get_user_details_by_accountid(account_id)

        # Assert - should return the data exactly as received from the API
        assert result is mock_user_data_cloud  # Same object reference
        assert isinstance(result, dict)
        assert all(
            key in result
            for key in ["accountId", "displayName", "email", "accountStatus"]
        )

    def test_deactivated_user_status_handling(self, users_mixin):
        """Test handling of deactivated users with status expansion."""
        # Arrange
        deactivated_user_data = {
            "accountId": "5b10ac8d82e05b22cc7d4ef5",
            "displayName": "Deactivated User",
            "accountStatus": "inactive",
            "status": "Deactivated",  # Expanded status
        }
        users_mixin.confluence.get_user_details_by_accountid.return_value = (
            deactivated_user_data
        )

        # Act
        result = users_mixin.get_user_details_by_accountid(
            "5b10ac8d82e05b22cc7d4ef5", expand="status"
        )

        # Assert
        assert result["accountStatus"] == "inactive"
        assert result["status"] == "Deactivated"
        users_mixin.confluence.get_user_details_by_accountid.assert_called_once_with(
            "5b10ac8d82e05b22cc7d4ef5", "status"
        )

    def test_method_delegation_to_confluence_client(
        self, users_mixin, mock_current_user_data
    ):
        """Test that methods properly delegate to the underlying confluence client."""
        # Test that the methods are thin wrappers around confluence client methods
        account_id = "test-account-id"
        username = "testuser"
        expand = "status"

        # Test account ID method delegation
        users_mixin.get_user_details_by_accountid(account_id, expand)
        users_mixin.confluence.get_user_details_by_accountid.assert_called_with(
            account_id, expand
        )

        # Test username method delegation
        users_mixin.get_user_details_by_username(username, expand)
        users_mixin.confluence.get_user_details_by_username.assert_called_with(
            username, expand
        )

        # Test current user method delegation - need to mock the return value
        users_mixin.confluence.get.return_value = mock_current_user_data
        users_mixin.get_current_user_info()
        users_mixin.confluence.get.assert_called_with("rest/api/user/current")



"""Tests for request logging utility."""

from datetime import timedelta
from pathlib import Path
from unittest.mock import MagicMock, patch

import pytest

from mcp_atlassian.utils.request_logging import (
    REQUESTS_LOG_FILE,
    install_request_logging,
    is_request_logging_enabled,
    log_request,
)


def test_log_request_writes_to_file(tmp_path, monkeypatch):
    """Test that log_request writes request info to the log file."""
    # Change to temp directory so log file is created there
    monkeypatch.chdir(tmp_path)

    # Create mock response
    mock_request = MagicMock()
    mock_request.method = "GET"
    mock_request.url = "https://example.com/api/test"

    mock_response = MagicMock()
    mock_response.request = mock_request
    mock_response.status_code = 200
    mock_response.elapsed = timedelta(milliseconds=150)

    # Call the log function
    log_request(mock_response)

    # Verify log file was created and contains expected content
    log_path = tmp_path / REQUESTS_LOG_FILE
    assert log_path.exists()

    content = log_path.read_text()
    assert "GET" in content
    assert "200" in content
    assert "150ms" in content
    assert "https://example.com/api/test" in content


def test_log_request_appends_multiple_requests(tmp_path, monkeypatch):
    """Test that multiple requests are appended to the log file."""
    monkeypatch.chdir(tmp_path)

    # Log two requests
    for i, (method, status) in enumerate([("GET", 200), ("POST", 201)]):
        mock_request = MagicMock()
        mock_request.method = method
        mock_request.url = f"https://example.com/api/{i}"

        mock_response = MagicMock()
        mock_response.request = mock_request
        mock_response.status_code = status
        mock_response.elapsed = timedelta(milliseconds=100)

        log_request(mock_response)

    log_path = tmp_path / REQUESTS_LOG_FILE
    lines = log_path.read_text().strip().split("\n")
    assert len(lines) == 2
    assert "GET" in lines[0]
    assert "POST" in lines[1]


def test_is_request_logging_enabled_false_by_default(monkeypatch):
    """Test that request logging is disabled by default."""
    monkeypatch.delenv("CONFLUENCE_LOG_REQUESTS", raising=False)
    assert is_request_logging_enabled() is False


def test_is_request_logging_enabled_true(monkeypatch):
    """Test that request logging can be enabled via env var."""
    monkeypatch.setenv("CONFLUENCE_LOG_REQUESTS", "true")
    assert is_request_logging_enabled() is True

    monkeypatch.setenv("CONFLUENCE_LOG_REQUESTS", "1")
    assert is_request_logging_enabled() is True

    monkeypatch.setenv("CONFLUENCE_LOG_REQUESTS", "yes")
    assert is_request_logging_enabled() is True


def test_install_request_logging_adds_hook(monkeypatch):
    """Test that install_request_logging adds the hook to session when enabled."""
    monkeypatch.setenv("CONFLUENCE_LOG_REQUESTS", "true")

    mock_session = MagicMock()
    mock_session.hooks = {"response": []}

    install_request_logging(mock_session)

    assert log_request in mock_session.hooks["response"]


def test_install_request_logging_skips_when_disabled(monkeypatch):
    """Test that install_request_logging does nothing when disabled."""
    monkeypatch.delenv("CONFLUENCE_LOG_REQUESTS", raising=False)

    mock_session = MagicMock()
    mock_session.hooks = {"response": []}

    install_request_logging(mock_session)

    assert log_request not in mock_session.hooks["response"]


def test_install_request_logging_no_duplicate_hooks(monkeypatch):
    """Test that hook is not added twice."""
    monkeypatch.setenv("CONFLUENCE_LOG_REQUESTS", "true")

    mock_session = MagicMock()
    mock_session.hooks = {"response": []}

    install_request_logging(mock_session)
    install_request_logging(mock_session)

    assert mock_session.hooks["response"].count(log_request) == 1


def test_log_request_handles_errors_gracefully(tmp_path, monkeypatch):
    """Test that logging errors don't crash the application."""
    monkeypatch.chdir(tmp_path)

    # Create a response that will cause an error
    mock_response = MagicMock()
    mock_response.request = None  # This will cause an AttributeError

    # Should not raise an exception
    log_request(mock_response)



from unittest.mock import MagicMock

import pytest

from mcp_atlassian.utils.decorators import check_write_access


class DummyContext:
    def __init__(self, read_only):
        self.request_context = MagicMock()
        self.request_context.lifespan_context = {
            "app_lifespan_context": MagicMock(read_only=read_only)
        }


@pytest.mark.asyncio
async def test_check_write_access_blocks_in_read_only():
    @check_write_access
    async def dummy_tool(ctx, x):
        return x * 2

    ctx = DummyContext(read_only=True)
    with pytest.raises(ValueError) as exc:
        await dummy_tool(ctx, 3)
    assert "read-only mode" in str(exc.value)


@pytest.mark.asyncio
async def test_check_write_access_allows_in_writable():
    @check_write_access
    async def dummy_tool(ctx, x):
        return x * 2

    ctx = DummyContext(read_only=False)
    result = await dummy_tool(ctx, 4)
    assert result == 8



"""Environment variable utility functions for MCP Atlassian."""

import os


def is_env_truthy(env_var_name: str, default: str = "") -> bool:
    """Check if environment variable is set to a standard truthy value.

    Considers 'true', '1', 'yes' as truthy values (case-insensitive).
    Used for most MCP environment variables.

    Args:
        env_var_name: Name of the environment variable to check
        default: Default value if environment variable is not set

    Returns:
        True if the environment variable is set to a truthy value, False otherwise
    """
    return os.getenv(env_var_name, default).lower() in ("true", "1", "yes")


def is_env_extended_truthy(env_var_name: str, default: str = "") -> bool:
    """Check if environment variable is set to an extended truthy value.

    Considers 'true', '1', 'yes', 'y', 'on' as truthy values (case-insensitive).
    Used for READ_ONLY_MODE and similar flags.

    Args:
        env_var_name: Name of the environment variable to check
        default: Default value if environment variable is not set

    Returns:
        True if the environment variable is set to a truthy value, False otherwise
    """
    return os.getenv(env_var_name, default).lower() in ("true", "1", "yes", "y", "on")


def is_env_ssl_verify(env_var_name: str, default: str = "true") -> bool:
    """Check SSL verification setting with secure defaults.

    Defaults to true unless explicitly set to false values.
    Used for SSL_VERIFY environment variables.

    Args:
        env_var_name: Name of the environment variable to check
        default: Default value if environment variable is not set

    Returns:
        True unless explicitly set to false values
    """
    return os.getenv(env_var_name, default).lower() not in ("false", "0", "no")


def get_custom_headers(env_var_name: str) -> dict[str, str]:
    """Parse custom headers from environment variable containing comma-separated key=value pairs.

    Args:
        env_var_name: Name of the environment variable to read

    Returns:
        Dictionary of parsed headers

    Examples:
        >>> # With CUSTOM_HEADERS="X-Custom=value1,X-Other=value2"
        >>> parse_custom_headers("CUSTOM_HEADERS")
        {'X-Custom': 'value1', 'X-Other': 'value2'}
        >>> # With unset environment variable
        >>> parse_custom_headers("UNSET_VAR")
        {}
    """
    header_string = os.getenv(env_var_name)
    if not header_string or not header_string.strip():
        return {}

    headers = {}
    pairs = header_string.split(",")

    for pair in pairs:
        pair = pair.strip()
        if not pair:
            continue

        if "=" not in pair:
            continue

        key, value = pair.split("=", 1)  # Split on first = only
        key = key.strip()
        value = value.strip()

        if key:  # Only add if key is not empty
            headers[key] = value

    return headers



"Tests for the date utility functions."

import pytest

from mcp_atlassian.utils import parse_date


def test_parse_date_invalid_input():
    """Test that parse_date returns an empty string for invalid dates."""
    with pytest.raises(ValueError):
        parse_date("invalid")


def test_parse_date_valid():
    """Test that parse_date returns the correct date for valid dates."""
    assert str(parse_date("2021-01-01")) == "2021-01-01 00:00:00"


def test_parse_date_epoch_as_str():
    """Test that parse_date returns the correct date for epoch timestamps as str."""
    assert str(parse_date("1612156800000")) == "2021-02-01 05:20:00+00:00"


def test_parse_date_epoch_as_int():
    """Test that parse_date returns the correct date for epoch timestamps as int."""
    assert str(parse_date(1612156800000)) == "2021-02-01 05:20:00+00:00"


def test_parse_date_iso8601():
    """Test that parse_date returns the correct date for ISO 8601."""
    assert str(parse_date("2021-01-01T00:00:00Z")) == "2021-01-01 00:00:00+00:00"


def test_parse_date_rfc3339():
    """Test that parse_date returns the correct date for RFC 3339."""
    assert (
        str(parse_date("1937-01-01T12:00:27.87+00:20"))
        == "1937-01-01 12:00:27.870000+00:20"
    )



"""Confluence space tools - get_spaces."""

import json
import logging
from typing import Annotated

from fastmcp import Context
from pydantic import Field

from mcp_atlassian.servers.dependencies import get_confluence_fetcher

from ._server import confluence_mcp

logger = logging.getLogger(__name__)


@confluence_mcp.tool(tags={"confluence", "read"})
async def get_spaces(
    ctx: Context,
    limit: Annotated[
        int,
        Field(description="Maximum number of spaces to return", ge=1, le=100, default=25),
    ] = 25,
) -> str:
    """List available Confluence spaces.

    Args:
        ctx: The FastMCP context.
        limit: Maximum number of spaces to return.

    Returns:
        JSON string with list of spaces.
    """
    confluence_fetcher = await get_confluence_fetcher(ctx)

    try:
        result = confluence_fetcher.get_spaces(start=0, limit=limit)
        spaces = []
        for space in result.get("results", []):
            spaces.append({
                "key": space.get("key"),
                "name": space.get("name"),
                "type": space.get("type"),
            })

        return json.dumps(
            {"success": True, "total": len(spaces), "spaces": spaces},
            indent=2,
            ensure_ascii=False,
        )

    except Exception as e:
        logger.error(f"Failed to get spaces: {e}")
        return json.dumps(
            {"error": f"Failed to get spaces: {str(e)}"},
            indent=2,
            ensure_ascii=False,
        )



"""
Common Confluence entity models.
This module provides Pydantic models for common Confluence entities like users
and attachments.
"""

import logging
import warnings
from typing import Any

from ..base import ApiModel
from ..constants import (
    UNASSIGNED,
)

logger = logging.getLogger(__name__)


class ConfluenceUser(ApiModel):
    """
    Model representing a Confluence user.
    """

    account_id: str | None = None
    display_name: str = UNASSIGNED
    email: str | None = None
    profile_picture: str | None = None
    is_active: bool = True
    locale: str | None = None

    @property
    def name(self) -> str:
        """
        Alias for display_name to maintain compatibility with tests.

        Deprecated: Use display_name instead.
        """
        warnings.warn(
            "The 'name' property is deprecated. Use 'display_name' instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        return self.display_name

    @classmethod
    def from_api_response(cls, data: dict[str, Any], **kwargs: Any) -> "ConfluenceUser":
        """
        Create a ConfluenceUser from a Confluence API response.

        Args:
            data: The user data from the Confluence API

        Returns:
            A ConfluenceUser instance
        """
        if not data:
            return cls()

        profile_pic = None
        if pic_data := data.get("profilePicture"):
            # Use the full path to the profile picture
            profile_pic = pic_data.get("path")

        return cls(
            account_id=data.get("accountId"),
            display_name=data.get("displayName", UNASSIGNED),
            email=data.get("email"),
            profile_picture=profile_pic,
            is_active=data.get("accountStatus") == "active",
            locale=data.get("locale"),
        )

    def to_simplified_dict(self) -> dict[str, Any]:
        """Convert to simplified dictionary for API response."""
        return {
            "display_name": self.display_name,
            "email": self.email,
            "profile_picture": self.profile_picture,
        }


class ConfluenceAttachment(ApiModel):
    """
    Model representing a Confluence attachment.
    """

    id: str | None = None
    type: str | None = None
    status: str | None = None
    title: str | None = None
    media_type: str | None = None
    file_size: int | None = None

    @classmethod
    def from_api_response(
        cls, data: dict[str, Any], **kwargs: Any
    ) -> "ConfluenceAttachment":
        """
        Create a ConfluenceAttachment from a Confluence API response.

        Args:
            data: The attachment data from the Confluence API

        Returns:
            A ConfluenceAttachment instance
        """
        if not data:
            return cls()

        return cls(
            id=data.get("id"),
            type=data.get("type"),
            status=data.get("status"),
            title=data.get("title"),
            media_type=data.get("extensions", {}).get("mediaType"),
            file_size=data.get("extensions", {}).get("fileSize"),
        )

    def to_simplified_dict(self) -> dict[str, Any]:
        """Convert to simplified dictionary for API response."""
        return {
            "id": self.id,
            "type": self.type,
            "status": self.status,
            "title": self.title,
            "media_type": self.media_type,
            "file_size": self.file_size,
        }



"""Configuration for integration tests."""

import pytest


def pytest_configure(config):
    """Add integration marker."""
    config.addinivalue_line(
        "markers", "integration: mark test as requiring integration with real services"
    )


def pytest_collection_modifyitems(config, items):
    """Skip integration tests unless explicitly requested."""
    if not config.getoption("--integration", default=False):
        # Skip integration tests by default
        skip_integration = pytest.mark.skip(reason="Need --integration option to run")
        for item in items:
            if "integration" in item.keywords:
                item.add_marker(skip_integration)


def pytest_addoption(parser):
    """Add integration option to pytest."""
    parser.addoption(
        "--integration",
        action="store_true",
        default=False,
        help="run integration tests",
    )



"""Confluence MCP server package.

This package contains the Confluence MCP server and tools organized into modules:
- _server.py: FastMCP instance and shared utilities
- sync.py: Space sync tools (sync_space)
- pages.py: Page tools (read_page, create_page, push_page_update)
- spaces.py: Space tools (get_spaces)
- comments.py: Comment and user tools (get_comments, add_comment, search_user)
- attachments.py: Attachment tools (download_attachments, upload_attachment, create_mermaid_diagram)
"""

# Import all tool modules to register them with confluence_mcp
from . import attachments, comments, pages, spaces, sync

# Export the MCP server instance and constants
from ._server import AUTO_FULL_SYNC_DAYS, confluence_mcp

# Re-export get_confluence_fetcher for backward compatibility (used by tests)
from mcp_atlassian.servers.dependencies import get_confluence_fetcher

# Export tool functions for backward compatibility (used by tests)
from .attachments import create_mermaid_diagram, download_attachments, upload_attachment
from .comments import add_comment, get_comments, search_user
from .pages import create_page, push_page_update, read_page
from .spaces import get_spaces
from .sync import sync_space, sync_space_impl

__all__ = [
    "confluence_mcp",
    "AUTO_FULL_SYNC_DAYS",
    "get_confluence_fetcher",
    # Sync tools
    "sync_space",
    "sync_space_impl",
    # Page tools
    "read_page",
    "create_page",
    "push_page_update",
    # Space tools
    "get_spaces",
    # Comment/user tools
    "get_comments",
    "add_comment",
    "search_user",
    # Attachment tools
    "download_attachments",
    "upload_attachment",
    "create_mermaid_diagram",
]
